<?php return array(
    'root' => array(
        'name' => 'icinga/icinga-php-thirdparty',
        'pretty_version' => 'dev-main',
        'version' => 'dev-main',
        'reference' => '44cd0d46123edffdb63e64560cf853f5bcf6d109',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'brick/math' => array(
            'pretty_version' => '0.9.3',
            'version' => '0.9.3.0',
            'reference' => 'ca57d18f028f84f777b2168cd1911b0dee2343ae',
            'type' => 'library',
            'install_path' => __DIR__ . '/../brick/math',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/block-react' => array(
            'pretty_version' => 'v1.5.0',
            'version' => '1.5.0.0',
            'reference' => '718b0571a94aa693c6fffc72182e87257ac900f3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/block-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/connection-manager-extra' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'reference' => '6ddf7da4cd75bf5aa54be13eea7be9aad9e97ada',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/connection-manager-extra',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/http-proxy-react' => array(
            'pretty_version' => 'v1.8.0',
            'version' => '1.8.0.0',
            'reference' => '09366dd3e13b36b90f8e47a6acaf5a2c96b79fb5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/http-proxy-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/mq-react' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => 'cab0147723017bc2deb3f248c607ad8e3c87e509',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/mq-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/redis-protocol' => array(
            'pretty_version' => 'v0.3.1',
            'version' => '0.3.1.0',
            'reference' => '271b8009887209d930f613ad3b9518f94bd6b51c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/redis-protocol',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/redis-react' => array(
            'pretty_version' => 'v2.6.0',
            'version' => '2.6.0.0',
            'reference' => 'f911455f9d7a77dd6f39c22548ddff521544b291',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/redis-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/soap-react' => array(
            'pretty_version' => 'v2.0.0',
            'version' => '2.0.0.0',
            'reference' => 'db2e0ba9b0380ac6c4a34df3f1021d2191d45d45',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/soap-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/socket-raw' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => '91e9f619f6769f931454a9882c21ffd7623d06cb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socket-raw',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/socks-react' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'reference' => 'eb3bd2ca5f43fea4b3bc4ef055179e0e6714730d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socks-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/stdio-react' => array(
            'pretty_version' => 'v2.6.0',
            'version' => '2.6.0.0',
            'reference' => 'dfa6c378aabdff718202d4e2453f752c38ea3399',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/stdio-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/term-react' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'reference' => 'eb6eb063eda04a714ef89f066586a2c49588f7ca',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/term-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/utf8-react' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'reference' => '8bc3f8c874cdf642c8f10f9ae93aadb8cd63da96',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/utf8-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'components/jquery' => array(
            'pretty_version' => '3.6.0',
            'version' => '3.6.0.0',
            'reference' => '6cf38ee1fd04b6adf8e7dda161283aa35be818c3',
            'type' => 'component',
            'install_path' => __DIR__ . '/../components/jquery',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'cweagans/composer-patches' => array(
            'pretty_version' => '1.7.3',
            'version' => '1.7.3.0',
            'reference' => 'e190d4466fe2b103a55467dfa83fc2fecfcaf2db',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../cweagans/composer-patches',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dompdf/dompdf' => array(
            'pretty_version' => 'v2.0.3',
            'version' => '2.0.3.0',
            'reference' => 'e8d2d5e37e8b0b30f0732a011295ab80680d7e85',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dompdf/dompdf',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'erusev/parsedown' => array(
            'pretty_version' => '1.7.4',
            'version' => '1.7.4.0',
            'reference' => 'cb17b6477dfff935958ba01325f2e8a2bfa6dab3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../erusev/parsedown',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'evenement/evenement' => array(
            'pretty_version' => 'v3.0.2',
            'version' => '3.0.2.0',
            'reference' => '0a16b0d71ab13284339abb99d9d2bd813640efbc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../evenement/evenement',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ezyang/htmlpurifier' => array(
            'pretty_version' => 'v4.16.0',
            'version' => '4.16.0.0',
            'reference' => '523407fb06eb9e5f3d59889b3978d5bfe94299c8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ezyang/htmlpurifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fig/http-message-util' => array(
            'pretty_version' => '1.1.5',
            'version' => '1.1.5.0',
            'reference' => '9d94dc0154230ac39e5bf89398b324a86f63f765',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fig/http-message-util',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/guzzle' => array(
            'pretty_version' => '7.8.0',
            'version' => '7.8.0.0',
            'reference' => '1110f66a6530a40fe7aea0378fe608ee2b2248f9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/guzzle',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/promises' => array(
            'pretty_version' => '2.0.1',
            'version' => '2.0.1.0',
            'reference' => '111166291a0f8130081195ac4556a5587d7f1b5d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/promises',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.6.1',
            'version' => '2.6.1.0',
            'reference' => 'be45764272e8873c72dbe3d2edcfdfcc3bc9f727',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'icinga/icinga-php-thirdparty' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => '44cd0d46123edffdb63e64560cf853f5bcf6d109',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-color-output' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => '2673074597eca9682d2fdfaee39a22418d4cc2f6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-color-output',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-diff' => array(
            'pretty_version' => '6.10.14',
            'version' => '6.10.14.0',
            'reference' => '38b6e816badaa97f75541484d6c3d7c3e5dbca78',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-diff',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-mb-string' => array(
            'pretty_version' => '1.4.8',
            'version' => '1.4.8.0',
            'reference' => 'ef8926ff849863bfea234e99ee827947bedd86b0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-mb-string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-sequence-matcher' => array(
            'pretty_version' => '3.2.10',
            'version' => '3.2.10.0',
            'reference' => '650164598be2c6ad6891dbd41de4cb5cc21cc91b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-sequence-matcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'masterminds/html5' => array(
            'pretty_version' => '2.8.1',
            'version' => '2.8.1.0',
            'reference' => 'f47dcf3c70c584de14f21143c55d9939631bc6cf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../masterminds/html5',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phenx/php-font-lib' => array(
            'pretty_version' => '0.5.4',
            'version' => '0.5.4.0',
            'reference' => 'dd448ad1ce34c63d09baccd05415e361300c35b4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phenx/php-font-lib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phenx/php-svg-lib' => array(
            'pretty_version' => '0.5.0',
            'version' => '0.5.0.0',
            'reference' => '76876c6cf3080bcb6f249d7d59705108166a6685',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phenx/php-svg-lib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'predis/predis' => array(
            'pretty_version' => 'v2.2.2',
            'version' => '2.2.2.0',
            'reference' => 'b1d3255ed9ad4d7254f9f9bba386c99f4bb983d1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../predis/predis',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => '0955afe48220520692d2d09f7ab7e0f93ffd6a31',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => 'e616d01114759c4c489f93b099585439f795fe35',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '1.1',
            'version' => '1.1.0.0',
            'reference' => 'cb6ce4845ce34a8ad9e68117c10ee90a29919eba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/collection' => array(
            'pretty_version' => '1.1.4',
            'version' => '1.1.4.0',
            'reference' => 'ab2237657ad99667a5143e32ba2683c8029563d4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/collection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/uuid' => array(
            'pretty_version' => '4.2.3',
            'version' => '4.2.3.0',
            'reference' => 'fc9bb7fb5388691fd7373cd44dcb4d63bbcf24df',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/cache' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'reference' => 'd47c472b64aa5608225f47965a484b75c7817d5b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/child-process' => array(
            'pretty_version' => 'v0.6.5',
            'version' => '0.6.5.0',
            'reference' => 'e71eb1aa55f057c7a4a0d08d06b0b0a484bead43',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/child-process',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/datagram' => array(
            'pretty_version' => 'v1.9.0',
            'version' => '1.9.0.0',
            'reference' => '13c259bb2fd04f238eeb85f3783167eedec46e2c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/datagram',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/dns' => array(
            'pretty_version' => 'v1.11.0',
            'version' => '1.11.0.0',
            'reference' => '3be0fc8f1eb37d6875cd6f0c6c7d0be81435de9f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/dns',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/event-loop' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'reference' => '6e7e587714fff7a83dcc7025aee42ab3b265ae05',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/event-loop',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/http' => array(
            'pretty_version' => 'v1.9.0',
            'version' => '1.9.0.0',
            'reference' => 'bb3154dbaf2dfe3f0467f956a05f614a69d5f1d0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/http',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise' => array(
            'pretty_version' => 'v2.10.0',
            'version' => '2.10.0.0',
            'reference' => 'f913fb8cceba1e6644b7b90c4bfb678ed8a3ef38',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise-stream' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => '87d1399a33926fd5ba28a7647346b71482787502',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise-stream',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise-timer' => array(
            'pretty_version' => 'v1.10.0',
            'version' => '1.10.0.0',
            'reference' => '4cb85c1c2125390748e3908120bb82feb99fe766',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise-timer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/socket' => array(
            'pretty_version' => 'v1.14.0',
            'version' => '1.14.0.0',
            'reference' => '21591111d3ea62e31f2254280ca0656bc2b1bda6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/socket',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/stream' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'reference' => '6fbc9672905c7d5a885f2da2fc696f65840f4a66',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/stream',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'rhumsaa/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '4.2.3',
            ),
        ),
        'ringcentral/psr7' => array(
            'pretty_version' => '1.3.0',
            'version' => '1.3.0.0',
            'reference' => '360faaec4b563958b673fb52bbe94e37f14bc686',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ringcentral/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabberworm/php-css-parser' => array(
            'pretty_version' => '8.4.0',
            'version' => '8.4.0.0',
            'reference' => 'e41d2140031d533348b2192a83f02d8dd8a71d30',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabberworm/php-css-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'shardj/zf1-future' => array(
            'pretty_version' => '1.23.5',
            'version' => '1.23.5.0',
            'reference' => '0464ee916ca73142ab733d80c83210d89cba3936',
            'type' => 'library',
            'install_path' => __DIR__ . '/../shardj/zf1-future',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v2.5.2',
            'version' => '2.5.2.0',
            'reference' => 'e8b495ea28c1d97b5e0c121748d6f9b53d075c66',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.28.0',
            'version' => '1.28.0.0',
            'reference' => 'ea208ce43cbb04af6867b4fdddb1bdbf84cc28cb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.28.0',
            'version' => '1.28.0.0',
            'reference' => '42292d99c55abe617799667f454222c54c60e229',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php80' => array(
            'pretty_version' => 'v1.28.0',
            'version' => '1.28.0.0',
            'reference' => '6caa57379c4aec19c0a12a38b59b26487dcfe4b5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php80',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php81' => array(
            'pretty_version' => 'v1.28.0',
            'version' => '1.28.0.0',
            'reference' => '7581cd600fa9fd681b797d00b02f068e2f13263b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php81',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tedivm/jshrink' => array(
            'pretty_version' => 'v1.6.8',
            'version' => '1.6.8.0',
            'reference' => '35a83e0ab661d6130da5930c0c4eafcc663b8cec',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tedivm/jshrink',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'wikimedia/less.php' => array(
            'pretty_version' => 'v3.2.1',
            'version' => '3.2.1.0',
            'reference' => '0d5b30ba792bdbf8991a646fc9c30561b38a5559',
            'type' => 'library',
            'install_path' => __DIR__ . '/../wikimedia/less.php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'zendframework/zendframework1' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '>=1.12.20',
            ),
        ),
    ),
);
