<?php

/*
 * This file is part of the Predis package.
 *
 * (c) 2009-2020 Daniele Alessandri
 * (c) 2021-2023 Till Krüss
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Predis\Command\Redis\Search;

use Predis\Command\Command as RedisCommand;

/**
 * @see https://redis.io/commands/ft.profile/
 *
 * Perform a FT.SEARCH or FT.AGGREGATE command and collects performance information.
 */
class FTPROFILE extends RedisCommand
{
    public function getId()
    {
        return 'FT.PROFILE';
    }

    public function setArguments(array $arguments)
    {
        [$index, $arguments] = $arguments;

        parent::setArguments(array_merge(
            [$index],
            $arguments->toArray()
        ));
    }
}
